@@ -497,6 +497,17 @@ impl hal::Instance<crate::Backend> for Instance {
             wl_egl_window_destroy(wl_window)
         }
     }
+
+    unsafe fn create_display_plane_surface(
+        &self,
+        _display_plane: &hal::display::DisplayPlane<crate::Backend>,
+        _plane_stack_index: u32,
+        _transformation: hal::display::SurfaceTransform,
+        _alpha: hal::display::DisplayPlaneAlpha,
+        _image_extent: hal::window::Extent2D,
+    ) -> Result<Surface, hal::display::DisplayPlaneSurfaceError> {
+        unimplemented!();
+    }
 }
 
 #[derive(Debug)]
